#if !defined(JUDOCA_CLASS)
#define JUDOCA_CLASS

#include <stdio.h>
#include <math.h>
#include <vector>
#include "ipl.h"
#include "highgui.h"
#include "derivatives.h"
#include "convert.h"

class Junction {

  public:
    int x,y;
    int n;
    double *orientation;
    float *strenght;

    Junction(int n) : n(n) {

      orientation= new double[n];
      strenght= new float[n];
    }

    Junction(const Junction& j) {

      n= j.n;
      x= j.x;
      y= j.y;

      orientation= new double[n];
      strenght= new float[n];

      for (int i=0; i<n; i++) {

        orientation[i]= j.orientation[i];
        strenght[i]= j.strenght[i];
      }
    }

	Junction& operator=(const Junction& j) {


      n= j.n;
      x= j.x;
      y= j.y;

      delete[] orientation;
      delete[] strenght;

      orientation= new double[n];
      strenght= new float[n];

      for (int i=0; i<n; i++) {

        orientation[i]= j.orientation[i];
        strenght[i]= j.strenght[i];
      }

	  return *this;
	}

    ~Junction() {

      delete[] orientation;
      delete[] strenght;
    }
};

int** getCirclePoints(double r, int *np);
int**** getLinePoints(int **c, int np, int r);
void deleteLinePoints(int**** pt, int np, int r);
void deleteCirclePoints(int **c);
IplImage* judoca(IplImage* B, IplImage* Bp, int radius, float sj, float minang, float maxangle2, std::vector<Junction> &junctions);
void saveJunctions(const char *name, std::vector<Junction> &junctions);
void loadJunctions(const char *name, std::vector<Junction> &junctions);
void drawJunctions(IplImage* image, std::vector<Junction> &junctions, int radius, int numberOfBranches= 0);
void nonMaxSupp(IplImage* junction_image, std::vector<Junction> &junctions, int radius);
IplImage* junctionRadialLines(int radius);
void saveJunctionHistogram(const char *name, std::vector<Junction> &junctions, int nangle, float awidth);


class Judoca {

  private:

	  IplImage* original;
	  IplImage* color;
      IplImage* juncbin;
      IplImage* junction;
      IplImage* edgemap;
      IplImage* edgeMaxBin;
      std::vector<Junction> junctions;

  public:

	Judoca(CString filename, bool display=true) {
    
	    color = cvvLoadImage( filename ); // load image
        junction= 0;
        juncbin= 0;
        edgemap= 0;
        edgeMaxBin= 0;

		if (display) {

		    cvvNamedWindow( "Original Image", 1 );  // create a window
		    cvvShowImage( "Original Image", color ); 
		 }

        original= cvCreateImage(cvSize(color->width,color->height),IPL_DEPTH_8U,1);
        iplColorToGray(color,original);
    }

    void detect(float variance, unsigned int edge_threshold,
                                int radius, float percentage, float minangle, float maxangle) {

      IplImage* imgf= convertToFloat(original); 
   
      IplImage* dx = derivex(imgf,variance);	
      IplImage* dy = derivey(imgf,variance);	
      IplImage* edge= norm(dx,dy);
  
      // apply threshold
      if (edgemap!=0)
        cvReleaseImage( &edgemap );
      edgemap= threshold(edge, edge_threshold, 100);

      // non max suppression
      IplImage* edgeMax= nonMax(edge, dx, dy);
      if (edgeMaxBin!=0)
        cvReleaseImage( &edgeMaxBin );
      edgeMaxBin= threshold(edgeMax, edge_threshold);

      // get junctions
      junctions.clear();
      if (juncbin!=0)
        iplDeallocate( juncbin,   IPL_IMAGE_ALL );
      juncbin= judoca(edgemap, edgeMaxBin, radius, percentage, minangle, maxangle, junctions);

      cvReleaseImage( &imgf );
      iplDeallocate( dx,         IPL_IMAGE_ALL );
      iplDeallocate( dy,         IPL_IMAGE_ALL );
      iplDeallocate( edge,       IPL_IMAGE_ALL );
      iplDeallocate( edgeMax,    IPL_IMAGE_ALL );
    }

    void display(int radius, int maxNumberOfBranches) {

      // display binary max edge map
  	  if ( edgemap!=0 && edgeMaxBin!=0) {
		IplImage* edgeview= combine(edgemap,edgeMaxBin);
		cvvNamedWindow( "Edges", 1 ); 
		cvvShowImage( "Edges", edgeview ); 
		// cvvSaveImage("testedges.bmp",convertToColor(edgeview));
		// cvvSaveImage("testradial.bmp",convertToColor(junctionRadialLines(radius)));
		cvReleaseImage( &edgeview );
	  }

      // display junctions
      if (junction!=0)
        iplDeallocate( junction,   IPL_IMAGE_ALL );
      junction= iplCloneImage(color);
      drawJunctions(junction,junctions,radius,maxNumberOfBranches);
      cvvNamedWindow( "Junctions", 1 ); 
      cvvShowImage( "Junctions", junction ); 
    }

    void suppress(int r) {

	  nonMaxSupp(juncbin,junctions,r);
    }

    void save(CString filename) {

      // save junctions
      saveJunctions(filename, junctions);
      CString name= filename.SpanExcluding("."); 
      name.Insert(name.GetLength(),".bmp");

      cvvSaveImage(name,junction);

      name= name.SpanExcluding("."); 
      name.Insert(name.GetLength(),".his");
	  saveJunctionHistogram(name, junctions, 36, 3.1415927*14./180.);	
    }

    void load(CString filename) {

      // load junctions from file
	  junctions.clear();
      loadJunctions(filename, junctions);
    }

    ~Judoca() {

		cvReleaseImage ( &color );
		cvReleaseImage ( &original );
        if (junction != 0) iplDeallocateImage( junction );
        if (juncbin != 0) iplDeallocateImage( juncbin );
		junction= juncbin= 0;
        if (edgemap != 0) cvReleaseImage( &edgemap );
        if (edgeMaxBin != 0) cvReleaseImage( &edgeMaxBin );
    }
};
	
#endif